//! Handhsake handling

pub mod dirsync;

use ::num_traits::FromPrimitive;

use crate::connection::{self, ProtocolVersion};

/// Handshake errors
#[derive(::thiserror::Error, Debug, Copy, Clone)]
pub enum Error {
    /// Error while parsing the handshake packet
    /// TODO: more detailed parsing errors
    #[error("not an handshake packet")]
    Parsing,
    /// No such Key ID
    #[error("unknown key id")]
    UnknownKeyID,
    /// Key error
    #[error("key: {0:?}")]
    Key(#[from] crate::enc::Error),
    /// Not enough data
    #[error("not enough data")]
    NotEnoughData,
}

pub(crate) struct HandshakeKey {
    pub id: crate::enc::asym::KeyID,
    pub key: crate::enc::asym::PrivKey,
}

/// Parsed handshake
#[derive(Debug, Clone)]
pub enum HandshakeData {
    /// Directory synchronized handhsake
    DirSync(dirsync::DirSync),
}

/// Kind of handshake
#[derive(::num_derive::FromPrimitive, Debug, Clone, Copy)]
#[repr(u8)]
pub enum Kind {
    /// 1-RTT, Directory synchronized handshake
    /// Request
    DirSyncReq = 0,
    /// 1-RTT, Directory synchronized handshake
    /// Response
    DirSyncResp,
    /*
    /// 2-RTT, Stateful connection
    Stateful,
    ...
    /// 3 RTT, Stateless connection
    Stateless,
    ....
    */
}

/// Parsed handshake
#[derive(Debug, Clone)]
pub struct Handshake {
    /// Fenrir Protocol version
    pub fenrir_version: ProtocolVersion,
    /// enum for the parsed data
    pub data: HandshakeData,
}

impl Handshake {
    const MIN_PKT_LEN: usize = 8;
    /// Parse the packet and return the parsed handshake
    pub fn parse(raw: &[u8]) -> Result<Self, Error> {
        if raw.len() < Self::MIN_PKT_LEN {
            return Err(Error::NotEnoughData);
        }
        let fenrir_version = match ProtocolVersion::from_u8(raw[0]) {
            Some(fenrir_version) => fenrir_version,
            None => return Err(Error::Parsing),
        };
        let handshake_kind = match Kind::from_u8(raw[1]) {
            Some(handshake_kind) => handshake_kind,
            None => return Err(Error::Parsing),
        };
        let data = match handshake_kind {
            Kind::DirSyncReq => dirsync::Req::parse(&raw[2..])?,
            Kind::DirSyncResp => dirsync::Resp::parse(&raw[2..])?,
        };
        Ok(Self {
            fenrir_version,
            data,
        })
    }
    pub(crate) fn work(&self, keys: &[HandshakeKey]) -> Result<(), Error> {
        todo!()
    }
}

trait HandshakeParsing {
    fn parse(raw: &[u8]) -> Result<HandshakeData, Error>;
}
